@@ -1,5 +1,5 @@
-from .. import LITEAPI_SUPPORTED_REQUEST_METHODS, BaseAPIRequest
-from ..BaseAPIRequest import APIMethod
+from .. import LITEAPI_SUPPORTED_REQUEST_METHODS
+from ..BaseAPIRequest import BaseAPIRequest,APIMethod
 
 def RequireAuth(authFunc):
     def AuthFunction(checkerFunc, **kwargs):
@@ -13,7 +13,7 @@ def RequireAuth(authFunc):
             elif callable(requestClassorMethod):
                 funname = requestClassorMethod.__name__
                 if funname.upper() in LITEAPI_SUPPORTED_REQUEST_METHODS:
-                    return APIMethod(methodFunc=requestClassorMethod, authFunc=authFunc(checkerFunc, **kwargs))
+                    return APIMethod.create(methodFunc=requestClassorMethod, authFunc=authFunc(checkerFunc, **kwargs))
 
         return inner
     return AuthFunction
\ No newline at end of file
