@@ -3,7 +3,7 @@ from secrets import token_urlsafe
 from re import findall
 from inspect import signature
 from .base_auth import RequireAuth
-from ..http_request import parse_unicode_fields
+from .._internals import _parse_unicode_fields
 from ..exception import UNAUTHORIZED_ERROR
 
 ALGORITHMS = {
@@ -33,13 +33,17 @@ def digest(**kwargs):
     if 'algorithm' not in kwargs \
     or 'ha1' not in kwargs \
     and ('username' not in kwargs \
-    or 'realm' not in kwargs \
-    or 'password' not in kwargs) \
-    or ('-sess' in kwargs['algorithm'] and 'cnonce' not in kwargs) \
+        or 'realm' not in kwargs \
+        or 'password' not in kwargs) \
+    or ('-sess' in kwargs['algorithm'] \
+        and 'cnonce' not in kwargs) \
     or 'nonce' not in kwargs \
     or 'uri' not in kwargs \
     or 'method' not in kwargs \
-    or ('qop' in kwargs and ('nc' not in kwargs or 'cnonce' not in kwargs)):
+    or ('qop' in kwargs \
+        and kwargs['qop'] in ('auth', 'auth-int') \
+        and ('nc' not in kwargs \
+            or 'cnonce' not in kwargs)):
         return
     
     hash = ALGORITHMS[kwargs['algorithm']]
@@ -49,43 +53,57 @@ def digest(**kwargs):
     ha2 = hash('{}:{}'.format(kwargs['method'], kwargs['uri']).encode()).hexdigest()
     response = None
     response = hash('{}:{}:'.format(ha1, kwargs['nonce']).encode())
-    if('qop' in kwargs):
+    if('qop' in kwargs and kwargs['qop'] in ('auth', 'auth-int')):
         response.update('{}:{}:{}:'.format(kwargs['nc'], kwargs['cnonce'], kwargs['qop']).encode())
+        if '-int' in kwargs['qop']:
+            ebody=hash(kwargs['entity-body']).hexdigest() if 'entity-body' in kwargs else ''
+            response.update('{}:'.format(ebody).encode())
     response.update(ha2.encode())
     return response.hexdigest()
 
 def doDigestAuth(checkerFunc, **kwargs):
     AUTH_SCHEME='Digest'
-    options = ' algorithm={}'.format(kwargs['algorithm'] if 'algorithm' in kwargs else 'MD5')
-    options += ', realm="{}"'.format(kwargs['realm']) if 'realm' in kwargs else ''
-    options += ', domain="{}"'.format(kwargs['domain']) if 'domain' in kwargs else ''
-    options += ', qop="{}"'.format(kwargs['qop']) if 'qop' in kwargs else ''
-    options += ', opaque="{}"'.format(kwargs['opaque']) if 'opaque' in kwargs else ''
+    options = []
+    if 'realm' in kwargs:
+        options.append(f'realm="{kwargs["realm"]}"')
+    if 'algorithm' in kwargs:
+        options.append(f'algorithm={kwargs["algorithm"]}')
+    if 'domain' in kwargs:
+        options.append(f'domain="{kwargs["domain"]}"')
+    if 'qop' in kwargs:
+        options.append(f'qop="{kwargs["qop"]}"')
+    if 'opaque' in kwargs:
+        options.append(f'opaque="{kwargs["opaque"]}"')
     nonce_handle = kwargs['nonce_handle'] if 'nonce_handle' in kwargs else token_urlsafe
     def digestAuth(self, **kwargs):
-        auth_options = options
+        auth_options = ' ' + ', '.join(options)
         if 'Authorization' not in self.request.headers:
-            auth_options += ', nonce="{}"'.format(nonce_handle())
-            raise UNAUTHORIZED_ERROR({'WWW-Authenticate': '{}{}'.format(AUTH_SCHEME, auth_options)})
+            auth_options += f', nonce="{nonce_handle()}"'
+            raise UNAUTHORIZED_ERROR({'WWW-Authenticate': f'{AUTH_SCHEME}{auth_options}'})
         ms = None
         if isinstance(self.request.headers['Authorization'], str):
-            ms = findall(r'Digest (?=.*\b(username\*?)\b=([^,$]+))(?=.*\b(realm)\b=([^,$]+))?(?=.*\b(uri)\b=([^,$]+))(?=.*\b(algorithm)\b=([^,$]+))(?=.*\b(nonce)\b=([^,$]+))(?=.*\b(nc)\b=([^,$]+))?(?=.*\b(cnonce)\b=([^,$]+))?(?=.*\b(qop)\b=([^,$]+))?(?=.*\b(response)\b=([^,$]+))(?=.*\b(opaque)\b=([^,$]+))?.+', self.request.headers['Authorization'])
+            ms = findall(r'[Dd][Ii][Gg][Ee][Ss][Tt] (?=.*\b(username\*?)\b=([^,$]+))(?=.*\b(realm)\b=([^,$]+))?(?=.*\b(uri)\b=([^,$]+))(?=.*\b(algorithm)\b=([^,$]+))(?=.*\b(nonce)\b=([^,$]+))(?=.*\b(nc)\b=([^,$]+))?(?=.*\b(cnonce)\b=([^,$]+))?(?=.*\b(qop)\b=([^,$]+))?(?=.*\b(response)\b=([^,$]+))(?=.*\b(opaque)\b=([^,$]+))?.+', self.request.headers['Authorization'])
         if not ms:
-            auth_options += ', nonce="{}"'.format(nonce_handle())
-            raise UNAUTHORIZED_ERROR({'WWW-Authenticate': '{}{}'.format(AUTH_SCHEME, auth_options)})
+            auth_options += f', nonce="{nonce_handle()}"'
+            raise UNAUTHORIZED_ERROR({'WWW-Authenticate': f'{AUTH_SCHEME}{auth_options}'})
         params = {
             'method': self.request.method
         }
         for i in range(0, len(ms[0]), 2):
             if ms[0][i]:
-                name, val = parse_unicode_fields(ms[0][i], ms[0][i + 1].strip('"'))
+                name, val = _parse_unicode_fields(ms[0][i], ms[0][i + 1].strip('"'))
                 params[name] = val
+        if 'entity-body' in self.request.headers:
+            params['entity-body'] = self.request.headers['entity-body']
         params['digest_password'] = lambda password: digest(password=password, **params)
         params['digest_ha1'] = lambda ha1: digest(ha1=ha1, **params)
         if not checkerFunc(**{k:v for k,v in params.items() if k in signature(checkerFunc).parameters}):
-            auth_options += ', nonce="{}"'.format(nonce_handle())
-            raise UNAUTHORIZED_ERROR({'WWW-Authenticate': '{}{}'.format(AUTH_SCHEME, auth_options)})
+            auth_options += f', nonce="{nonce_handle()}"'
+            raise UNAUTHORIZED_ERROR({'WWW-Authenticate': f'{AUTH_SCHEME}{auth_options}'})
         return {'username': params['username']}
+    digestAuth.__name__ = 'digest'
+    digestAuth._checker = checkerFunc
+    digestAuth._args = kwargs
     return digestAuth
 
 RequireDigestAuth = RequireAuth(doDigestAuth)
\ No newline at end of file
